@@ -114,16 +114,15 @@ namespace Beagle.Daemon.GaimLogQueryable {
 		/////////////////////////////////////////////////
 
 		// Sets up an Inotify watch on all subdirectories withing ~/.gaim/logs
-		private int Watch (string path)
+		private void Watch (string path)
 		{
 			DirectoryInfo root = new DirectoryInfo (path);
 			
 			if (! root.Exists) {
 				log.Warn ("IM: {0} cannot watch path. It doesn't exist.", path);
-				return 0;	
+				return;	
 			}
 			
-			int file_count = 0;
 			Queue queue = new Queue ();
 			queue.Enqueue (root);
 
@@ -132,8 +131,7 @@ namespace Beagle.Daemon.GaimLogQueryable {
 				
 				// Setup watches on the present directory.
 				int wd = Inotify.Watch (dir.FullName,
-							Inotify.EventType.CreateSubdir
-							| Inotify.EventType.Modify);
+							Inotify.EventType.Create | Inotify.EventType.Modify);
 				
 				watched [wd] = true;
 
@@ -141,8 +139,6 @@ namespace Beagle.Daemon.GaimLogQueryable {
 				foreach (DirectoryInfo subdir in dir.GetDirectories ())
 					queue.Enqueue (subdir);
 			}
-			
-			return file_count;
 		}
 		
 		/////////////////////////////////////////////////
@@ -162,23 +158,22 @@ namespace Beagle.Daemon.GaimLogQueryable {
 		private void OnInotifyEvent (int wd,
 					     string path,
 					     string subitem,
-					     Inotify.EventType type,
-					     uint cookie)
+					     string srcpath,
+					     Inotify.EventType type)
 		{
 			if (subitem == "" || ! watched.Contains (wd))
 				return;
 
 			string full_path = Path.Combine (path, subitem);
 
-			switch (type) {
-				
-				case Inotify.EventType.CreateSubdir:
-					Watch (full_path);
-					break;
-	
-				case Inotify.EventType.Modify:
-					IndexLog (full_path, Scheduler.Priority.Immediate);
-					break;
+			if ((type & Inotify.EventType.Create) != 0 && (type & Inotify.EventType.IsDirectory) != 0) {
+				Watch (full_path);
+				return;
+			}
+
+			if ((type & Inotify.EventType.Modify) != 0) {
+				IndexLog (full_path, Scheduler.Priority.Immediate);
+				return;
 			}
 		}
 
@@ -190,11 +185,6 @@ namespace Beagle.Daemon.GaimLogQueryable {
 			indexable.Timestamp = log.Timestamp;
 			indexable.Type = "IMLog";
 
-			// We don't have a specific mime type for this
-			// blob, but a mime type must be specified for
-			// indexables that provide a stream
-			indexable.MimeType = "text/plain";
-			
 			StringBuilder text = new StringBuilder ();
 			foreach (ImLog.Utterance utt in log.Utterances) {
 				//Console.WriteLine ("[{0}][{1}]", utt.Who, utt.Text);
@@ -209,6 +199,9 @@ namespace Beagle.Daemon.GaimLogQueryable {
 			indexable.AddProperty (Property.NewKeyword ("fixme:identity", log.Identity));
 			indexable.AddProperty (Property.NewDate ("fixme:endtime", log.EndTime));
 
+			indexable.AddProperty (Property.New ("fixme:client", log.Client));
+			indexable.AddProperty (Property.New ("fixme:protocol", log.Protocol));
+
 			StringReader reader = new StringReader (text.ToString ());
 			indexable.SetTextReader (reader);
 			
@@ -242,7 +235,7 @@ namespace Beagle.Daemon.GaimLogQueryable {
 							       "fixme:endtime", "fixme:starttime");
 		}
 
-		override public string GetSnippet (QueryBody body, Hit hit)
+		override public string GetSnippet (string[] query_terms, Hit hit)
 		{
 			// FIXME: This does the wrong thing for old-style logs.
 			string file = hit ["fixme:file"];
@@ -254,25 +247,37 @@ namespace Beagle.Daemon.GaimLogQueryable {
 			if (log == null)
 				return null;
 
+			string result = "";
+
 			// FIXME: This is very lame, and doesn't do the
 			// right thing w/ stemming, word boundaries, etc.
 			foreach (ImLog.Utterance utt in log.Utterances) {
-				string lower_text = utt.Text.ToLower ();
 				string text = utt.Text;
-				int i = -1;
-				foreach (string query_text in body.Text) {
-					i = lower_text.IndexOf (query_text.ToLower ());
-					if (i >= 0) {
-						text = String.Concat (text.Substring (0, i), "<b>", text.Substring (i, query_text.Length), "</b>", text.Substring (i + query_text.Length));
-						lower_text = text.ToLower ();
-					}
-				}
-				if (i >= 0)
-					return text;
+				string who = utt.Who;
+				
+				string snippet = SnippetFu.GetSnippet (query_terms, new StringReader (text));
+
+				if (snippet == null || snippet == "")
+					continue;
+
+				result += String.Format ("{0}: {1} ", who, snippet);
+
+				if (result.Length > 300)
+					break;
 			}
 
-			// If all else fails, return the log's generic snippet
-			return log.Snippet;
+			if (result != "")
+				return result;
+			else
+				return log.Snippet;
+		}
+
+		override protected bool HitIsValid (Uri uri)
+		{
+			if (File.Exists (uri.LocalPath))
+				return true;
+			
+			return false;
 		}
 	}
 }
